;; gingerbread_house
;;
;; Copyright 2016 - By Michael Kohn
;; http://www.mikekohn.net/
;; mike to mikekohn.net
;;
;; Control 2 strings of LEDs and 2 servo motors.

.include "msp430x2xx.inc"

RAM equ 0x0200

SERVO_0_RESET equ RAM
SERVO_1_RESET equ RAM+2

.macro PAUSE(count)
.scope
  mov.w #count, r14
pause:
  cmp.w #0, r14
  jnz pause
.ends
.endm

;  r4 = Servo 0 current value
;  r5 = Servo 1 currnet value
;  r6 = Servo 20ms counter
;  r7 = Servo state counter
;  r8 =
;  r9 =
; r10 =
; r11 =
; r12 =
; r13 =
; r14 = interrupt count
; r15 =

  .org 0xf800
start:
  ;; Turn off watchdog
  mov.w #(WDTPW|WDTHOLD), &WDTCTL

  ;; Turn off interrupts
  dint

  ;; Set up stack pointer
  mov.w #0x0280, SP

  ;; Set MCLK to 16 MHz with DCO 
  mov.b #DCO_7, &DCOCTL
  mov.b #RSEL_14, &BCSCTL1
  mov.b #0, &BCSCTL2

  ;; Set up output pins
  ;; P1.0 = LED string 0
  ;; P1.1 = LED string 1
  ;; P1.2 = Servo 0 (reverse logic)
  ;; P1.3 = Servo 1 (reverse logic)
  mov.b #0x0f, &P1DIR
  mov.b #0x0c, &P1OUT
  ;mov.b #0x40, &P2DIR
  ;mov.b #0x00, &P2OUT
  ;mov.b #0x00, &P2SEL

  ;; Set up Timer
  mov.w #1600, &TACCR0
  mov.w #(TASSEL_2|MC_1), &TACTL ; SMCLK, DIV1, COUNT to TACCR0
  mov.w #CCIE, &TACCTL0
  mov.w #0, &TACCTL1

  mov.w #10, &SERVO_0_RESET
  mov.w #10, &SERVO_1_RESET
  mov.w &SERVO_0_RESET, r4
  mov.w &SERVO_1_RESET, r5
  mov.w #200, r6
  mov.w #0, r14
  mov.w #0, r7

  ;; Enable interrupts
  eint

main:
  PAUSE(10000)
  bic.b #1, &P1OUT
  bis.b #2, &P1OUT
  PAUSE(10000)
  bis.b #1, &P1OUT
  bic.b #2, &P1OUT
  inc.b r7
  cmp.w #2, r7
  jz servo_state_1
  cmp.w #4, r7
  jz servo_state_2
  cmp.w #6, r7
  jz servo_state_3
  cmp.w #8, r7
  jz servo_state_0
  jmp main

servo_state_1:
  mov.w #20, &SERVO_0_RESET
  jmp main

servo_state_2:
  mov.w #20, &SERVO_1_RESET
  jmp main

servo_state_3:
  mov.w #10, &SERVO_0_RESET
  jmp main

servo_state_0:
  mov.w #10, &SERVO_1_RESET
  mov.w #0, r7
  jmp main

timer_interrupt:
  dec.w r14
  dec.w r6
  jnz dont_reset
  mov.w &SERVO_0_RESET, r4
  mov.w &SERVO_1_RESET, r5
  mov.w #200, r6
  bis.b #0x0c, &P1OUT
dont_reset:
  cmp.w r4, r6
  jnz dont_update_servo0
  bic.b #0x04, &P1OUT
dont_update_servo0:
  cmp.w r5, r6
  jnz dont_update_servo1
  bic.b #0x08, &P1OUT
dont_update_servo1:
  reti 

  org 0xffe8
vectors:
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0
  dw timer_interrupt       ; Timer_A2 TACCR0, CCIFG
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0
  dw start                 ; Reset



